/*
    addcol:  Routine to insert blank columns into an ASCII file.  The file is
    assumed to have fixed-length records.

    Format

       addcol <input_file> <column>[rep] [ <column>[rep] ... ]

    where "column" is the column number in the input record and "rep" is
    a repetition factor of the form: 

         <single non-digit, non-blank character><positive integer>

    Blank columns will be inserted AFTER the column numbers listed in the 
    command line.   All columns numbers are taken with reference to the 
    original input record.  So, for example, if the input record contains
    5 columns, then

       addcol <input file> 0 3x2 5

    will result in an output file with blank columns at column 1, 5, 6 and 9.
    Blank columns at the end of record will always be inserted BEFORE the 
    linefeed record delimiter.

    06 Oct 1993, acr
    15 Apr 2011, acr: Modified calls to exit() to use EXIT_SUCCESS and 
                      EXIT_FAILURE rather than specific error codes.

*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#define MAX_RECORD_LENGTH 10000
#define MAX_COLUMNS         100

void b2sort();

main (argc,argv)
  int   argc;
  char *argv[];

{ int     i,j,k;                       /* loop/subscript */
  char    inptline[MAX_RECORD_LENGTH]; /* input record */
  char    outline[MAX_RECORD_LENGTH];  /* output record */
  int     column[MAX_COLUMNS];         /* columns to be inserted */
  int     counts[MAX_COLUMNS];         /* number of columns to be inserted */
  int     rcount;                      /* record count */
  int     imaxlength;                  /* maximum input record length */
  int     iminlength;                  /* minimum input record length */
  int     omaxlength;                  /* maximum output record length */
  int     ominlength;                  /* minimum output record length */
  int     length;                      /* column list subscript */
  int     c;                           /* column list subscript */
  FILE   *fp;                          /* input file pointer */
  char   *string;                      /* string pointer for length checking */
  int     numc;                        /* number of columnsi n input list */


  /* First, make sure there are at least three arguments, then open
     the input file:
  */

  if (argc < 3)
    { fprintf (stderr,"Usage:  addcol <input file> <column> [<column> ...]\n");
      exit(EXIT_SUCCESS);
    }

  if (argc > 102)
    { fprintf (stderr,"addcol: maximum number of columns is 100.\n");
      exit(EXIT_FAILURE);
    }

  if ((fp = fopen(argv[1],"r"))==NULL)
    { fprintf (stderr,"addcol: Unable to open %s for reading.\n",argv[1]);
      exit(EXIT_FAILURE);
    }


  /* Collect the list of columns to be skipped and sort it: */

  numc = 0;
  for (i=2; i< argc; ++i)
    { column[numc] = strtol(argv[i], &string, 10);
      if (column[numc] < 0)
          fprintf (stderr, 
                  "addcol: Invalid column number (%s) ignored.\n",argv[i]);
      else
        { if (strlen(string) == 0)    /* no repetition specified */
              counts[numc] = 1;
          else
            { string[0]=' ';     /* remove the "*" repetition flag */
              counts[numc] = atoi(string);
              if (counts[c] < 0)
                { fprintf (stderr,
                          "addcol: Invalid repetition factor (%s) ignored.\n",
                           string);
                  counts[c] = 1;
                }
            }
          ++numc;
        }
    }
  b2sort(column,counts,numc);


  /* Loop through all records in the file: */

  imaxlength = 0;
  iminlength = MAX_RECORD_LENGTH;
  omaxlength = 0;
  ominlength = MAX_RECORD_LENGTH;
  rcount    = 0;

  while ((string=fgets(inptline,MAX_RECORD_LENGTH,fp)) != NULL)
    { ++rcount;
      length = strlen(string);
      if (length > MAX_RECORD_LENGTH)
         { fprintf (stderr,
                    "delcol: Record %d length (%d) exceeds maximum (%d)\n",
                    rcount, strlen(string), MAX_RECORD_LENGTH);
           exit(EXIT_FAILURE);
         }

      imaxlength = (length > imaxlength) ? length : imaxlength;
      iminlength = (length < iminlength) ? length : iminlength;

      /* Remove the newline (\n) from the end of the input string: */

      --length;
      inptline[length] = '\0';

      /* Set up subscripts for stepping through the various arrays: */

      c = j = 0;

      /* Loop through the characters in the record: */

      for (i=0; i<length; ++i)
        { while ((c < numc) &&            /* find next applicable column */
                 (i > column[c])) ++c;
          if (i == column[c])             /* if this column is on the list */
              for (k=0; k<counts[c]; ++k) /*   insert blank columns        */
                 outline[j++] = ' ';
          outline[j++] = inptline[i];
        }

      /* At the end of record.  If there are still columns in the list, add
         blanks for each column before the record delimiter:
      */

      if ((i-1) == column[c]) ++c;
      while (c < numc) 
        { for (k=0; k<counts[c]; ++k) outline[j++] = ' ';
          ++c;
        }
      outline[j] = '\0';
      length = ++j;

      /* Write the result to the standard output: */

      omaxlength = (length > omaxlength) ? length : omaxlength;
      ominlength = (length < ominlength) ? length : ominlength;
      puts(outline);

    }


  /* Done.  Close file and print results: */

  fclose(fp);
  fprintf (stderr, "%d records processed\n",rcount);
  fprintf (stderr, "  Maximum input length: %5d\n",imaxlength);
  fprintf (stderr, "  Minimum input length: %5d\n",iminlength);
  fprintf (stderr, " Maximum output length: %5d\n",omaxlength);
  fprintf (stderr, " Minimum output length: %5d\n",ominlength);

  exit(EXIT_FAILURE);
}

/*==========================================================================*/

void b2sort(list1,list2,items)
  int list1[];
  int list2[];
  int items;

/* Bubble sort routine for integer array */

{ int  i,j;  /*loop/subscript */
  int  tmp;  /* holding place */

  for (i=0; i<items-1; ++i)
    { for (j=i+1; j<items; ++j)
        { if (list1[i] > list1[j])
            { tmp     = list1[i];
              list1[i] = list1[j];
              list1[j] = tmp;
              tmp     = list2[i];
              list2[i] = list2[j];
              list2[j] = tmp;
            }
        }
    }

  return;
}
