/*

  chkascii

  Routine to check every stinkin' character in a text file to see if it is
  in the ASCII character set.  Default behavior is to further limit this to
  the printing characters and record delimiters.

  Format:  chkascii [-iso] [-cr] [-ff] [text_file]

    where  -iso  allow all printing characters from the ISO/IEC 8859-1 
                 (Latin-1) code page.

           -cr   treats CR as the line delimiter rather than LF. (Needed
                 for unconverted Mac files.)

           -ff   Allow formfeed characters

  By default the only non-printing characters accepted are the blank and
  the CR and LF control codes, which must appear as line delimiters.

  Output is to STDOUT.  If no input file is supplied, input is read from 
  STDIN.

  2010-11-22  ACR:  Fixed stupid printing error for uc > 127

*/

#define RECORD_DELIMITER '\n'
#define CR     13
#define LF     10
#define CRLF 1310
#define TRUE    1
#define FALSE   0

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <errno.h>

main (int argc, char *argv[])

{ long int       linecount;    /* running count of input lines */
  long int       position;     /* character position in the line  */
  unsigned char  uc;           /* current character */
  unsigned char  last_uc;      /* last character */
  char           dlm;          /* record delimiter */
  long int       char_total;   /* total bad characters found */
  long int       line_total;   /* total lines with bad characters */

  int            ISO_ASCII;    /* TRUE if allowing all ISO-8859-1 printing */
  int            ff_ok;        /* TRUE if formfeed characters are allowed */
  FILE          *ifp;
  int            i;
  int            dlm_type;     /* 1310 = CR/LF, 10 = LF, 13 = CR */
  int            first_on_line;  /* true if no bad characters found yet */

  ISO_ASCII = FALSE;
  ff_ok     = FALSE;
  dlm       = '\n';
  dlm_type  = 0; 
  ifp        = NULL;

  char_total = 0;
  line_total = 0;
  last_uc    = 0;

 
  /* Parse arguments and options: */

  for (i=1; i<argc; i++)
    { if (strcmp(argv[i],"-iso") == 0)
        { ISO_ASCII = TRUE; }

      else if (strcmp(argv[i],"-cr") == 0)
        { dlm = '\r'; 
          dlm_type = CR;
        }

      else if (strcmp(argv[i],"-ff") == 0)
        { ff_ok = TRUE; }

      else
        { ifp = fopen(argv[i],"r");
          if (!ifp)
            { fprintf (stderr,"Could not open %s for reading (%s)\n",
                      argv[i],strerror(errno));
              exit(100);
            }
        }
    } /* for i */

  /* Make sure we've got input from somewhere: */

  if (ifp == NULL)
    { ifp = stdin; }


  /* Initialize the counters and loop through each character of the file: */

  linecount = 1;
  position  = 0;
  first_on_line = TRUE;
  last_uc   = 0;

  uc = fgetc(ifp);
  position++;

  while (!feof(ifp))
    { /* Deal with record delimiters: */

      if (uc == CR)
        { if (dlm_type == CR)
            { linecount++;           /* next line */
              position = 0;
              first_on_line = TRUE;
            }
          else if (dlm_type == LF)
            { fprintf(stdout,"Non-printing 7-bit character - ");
              fprintf(stdout,"line %d, character %d: ",linecount,position);
              fprintf(stdout,"code %u [0x%02X]\n", uc, uc);
              char_total++;
              if (first_on_line)
                { line_total++; 
                  first_on_line = FALSE;
                }
            }
          else 
            { uc = fgetc(ifp);
              if (uc == LF)
                { linecount++;        /* next line */
                  position = 0;
                  first_on_line = TRUE;
                  dlm_type = CRLF;
                }
              else
                { fprintf(stdout,"Non-printing 7-bit character - ");
                  fprintf(stdout,"line %d, character %d: ",linecount,position);
                  fprintf(stdout,"code %u [0x%02X]\n", uc, uc);
                  char_total++;
                  if (first_on_line)
                    { line_total++; 
                      first_on_line = FALSE;
                    }
                }
            }
        }  /* end if (uc == CR) */

      else if (uc == LF)
        { if (dlm_type == CR)
            { fprintf(stdout,"Non-printing 7-bit character - ");
              fprintf(stdout,"line %d, character %d: ",linecount,position);
              fprintf(stdout,"code %u [0x%02X]\n", uc, uc);
              char_total++;
              if (first_on_line)
                { line_total++; 
                  first_on_line = FALSE;
                }
            }

          /* If we are expecting CR/LF delimiters, then we should have 
             found the delimiter above, so if this test succeeds there
             is a problem:
          */

          else if (dlm_type == CRLF)
            { fprintf(stdout,"Non-printing 7-bit character - ");
              fprintf(stdout,"line %d, character %d: ",linecount,position);
              fprintf(stdout,"code %u [0x%02X]\n", uc, uc);
              fprintf(stdout," -- Possible inconsistent line delimiters\n");
              char_total++;
              if (first_on_line)
                { line_total++; 
                  first_on_line = FALSE;
                }
            }
          else 
            { linecount++;
              position = 0;
              first_on_line = TRUE;
              dlm_type = LF;
            }
        }


      /* PDS Standards allow form feed characters only in document files,
         so if we find one and we're not specifically accepting them, we
         flag it as a problem:
      */

      else if (uc == 12)
        { if (!ff_ok)
            { fprintf (stdout, "Formfeed character - ");
              fprintf (stdout, "line %d, character %d: ",linecount,position);
              fprintf (stdout, "code %u [0x%02X]\n",uc,uc);
              char_total++;
              if (first_on_line)
                { line_total++; 
                  first_on_line = FALSE;
                }
            }
        }

      /* Check for other non-printing (and by this point non-line-delimiting)
         characters:
      */

      else if (uc < 32 ||  uc == 127)
        { fprintf (stdout, "Non-printing 7-bit character - ");
          fprintf (stdout, "line %d, character %d: ",linecount,position);
          fprintf (stdout, "code %u [0x%02X]\n",uc,uc);
          char_total++;
          if (first_on_line)
            { line_total++; 
              first_on_line = FALSE;
            }
        }

      /* Now for non-7-bit characters (values > 127). If we're 
         allowing the ISO 8859-1 codepage characters we still reject
         non-printing ones:
      */

      else if (ISO_ASCII  &&  uc > 127)
        { if (uc > 0x7F  &&  uc < 0xA0)
            { fprintf (stdout, "Non-printing 8-bit (ISO) character found, ");
              fprintf (stdout, "line %d, character %d: ",linecount,position);
              fprintf (stdout, "code %u [0x%02X]\n",uc,uc);
              char_total++;
              if (first_on_line)
                { line_total++; 
                  first_on_line = FALSE;
                }
            }
        }

      /* If we're not allowing the ISO codepage, we reject all of them: */

      else if (uc > 127)
        { fprintf (stdout, "Invalid 8-bit character - ");
          fprintf (stdout, "line %d, character %d: ",linecount,position);
          fprintf (stdout, "code %u [0x%02X]\n",uc,uc);
          char_total++;
          if (first_on_line)
            { line_total++; 
              first_on_line = FALSE;
            }
        }
     

      /* Otherwise, next character... */

      last_uc = uc;
      uc = fgetc(ifp);
      position++;

    }  /* while (!eof) */

  /* If we ended on a line delimiter, we'll need to decrement the
     linecount:
  */

  if ( (dlm_type == CR  &&  last_uc == CR)  ||
       (dlm_type != CR  &&  last_uc == LF) )
    { linecount--; }

/* Debug stuff...
  fprintf (stdout," LINECOUNT: %d\n",linecount);
  fprintf (stdout,"CHAR_TOTAL: %d\n",char_total);
  fprintf (stdout,"LINE_TOTAL: %d\n",line_total);
*/

  /* If we didn't find any problem characters, we should say so: */

  if (char_total == 0)
    { if (ISO_ASCII)
        { fprintf (stdout,"No invalid ISO 8859-1 characters found.\n"); }
      else
        { fprintf (stdout,"No invalid 7-bit ASCII characters found.\n"); }
    }

  if (ifp != stdin)
    { close (ifp); }

  exit(0);
}
