/*

  delcol - Routine to delete columns from an ASCII dataset.

  Format

     delcol <input_file> <column> [ <column> ... ]

  All column numbers are as they appear in the unaltered record.  Ranges
  are not understood; output is to the standard output.  A maximum of 100
  column numbers can be specified.


  10 Sep 1993, acr:

*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#define MAX_RECORD_LENGTH 10000
#define MAX_COLUMNS         100

void bsort();

main(argc,argv)
  int   argc;
  char *argv[];

{ int    i,j,k;                       /* loop/subscript */
  char   inptline[MAX_RECORD_LENGTH]; /* input record */
  char   outline[MAX_RECORD_LENGTH];  /* output record */
  int    column[MAX_COLUMNS];         /* columns to be deletes */
  int    rcount;                      /* record count */
  int    imaxlength;                  /* maximum input record length */
  int    iminlength;                  /* minimum input record length */
  int    omaxlength;                  /* maximum output record length */
  int    ominlength;                  /* minimum output record length */
  int    length;                      /* current line length */
  int    c;                           /* column list subscript */
  FILE  *fp;                          /* input file pointer */
  char  *string;                      /* string pointer for length checking */
  int    numc;                        /* number of columns in input list */

  /* First, make sure there are at least three arguments, then open
     the input file:
  */

  if (argc < 3)
    { fprintf (stderr,"Usage:  delcol <input file> <column> [<column> ...]\n");
      exit(EXIT_SUCCESS);
    }

  if (argc > 102)
    { fprintf (stderr,"delcol: maximum number of columns is 100.\n");
      exit(EXIT_FAILURE);
    }

  if ((fp = fopen(argv[1],"r"))==NULL)
    { fprintf (stderr,"delcol: Unable to open %s for reading.\n",argv[1]);
      exit(EXIT_FAILURE);
    }


  /* Collect the list of columns to be skipped and sort it: */

  numc = 0;
  for (i=2; i< argc; ++i)
    { column[numc] = atoi(argv[i]) - 1;
      if (column[numc] < 0)
          fprintf (stderr, 
                  "delcol: Invalid column number (%s) ignored.\n",argv[i]);
      else
          ++numc;
    }
  bsort(column,numc);


  /* Loop through all records in the file: */

  imaxlength = 0;
  iminlength = MAX_RECORD_LENGTH;
  omaxlength = 0;
  ominlength = MAX_RECORD_LENGTH;
  rcount    = 0;

  while ((string=fgets(inptline,MAX_RECORD_LENGTH,fp)) != NULL)
    { ++rcount;
      length = strlen(string);
      if (length > MAX_RECORD_LENGTH)
         { fprintf (stderr,
                    "delcol: Record %d length (%d) exceeds maximum (%d)\n",
                    rcount, strlen(string), MAX_RECORD_LENGTH);
           exit(EXIT_FAILURE);
         }

      imaxlength = (length > imaxlength) ? length : imaxlength;
      iminlength = (length < iminlength) ? length : iminlength;

      /* Remove the newline (\n) from the end of the input string: */

      --length;
      inptline[length] = '\0';

      /* Set up subscripts for stepping through the various arrays: */

      c = j = 0;

      /* Loop through the characters in the record: */

      for (i=0; i<length; ++i)
        { while ((c < numc)  &&          /* find next higher column number */
                 (i > column[c])) ++c;
          if (i != column[c])            /* this column is not on the list */
            { outline[j] = inptline[i];
              ++j;
            }
        }
      outline[j] = '\0';     /* Add delimiter to end.  This is replaced by */
      length = ++j;          /* a newline (\N) on output.                  */

      /* Write the result to the standard output: */

      omaxlength = (length > omaxlength) ? length : omaxlength;
      ominlength = (length < ominlength) ? length : ominlength;
      puts(outline);

    }


  /* Done.  Close file and print results: */

  fclose(fp);
  fprintf (stderr, "%d records processed\n",rcount);
  fprintf (stderr, "  Maximum input length: %5d\n",imaxlength);
  fprintf (stderr, "  Minimum input length: %5d\n",iminlength);
  fprintf (stderr, " Maximum output length: %5d\n",omaxlength);
  fprintf (stderr, " Minimum output length: %5d\n",ominlength);

  exit(EXIT_SUCCESS);
}

/*==========================================================================*/

void bsort(list,items)
  int list[];
  int items;

/* Bubble sort routine for integer array */

{ int  i,j;  /*loop/subscript */
  int  tmp;  /* holding place */

  for (i=0; i<items-1; ++i)
    { for (j=i+1; j<items; ++j)
        { if (list[i] > list[j])
            { tmp     = list[i];
              list[i] = list[j];
              list[j] = tmp;
            }
        }
    }

  return;
}
