/* 
   findtab

   Routine to find tab and other non-printing characters in ASCII text files,
   excluding carriage-control characters. The location of the non-printing
   characters is reported on stdout. If more than MAX_ERRORS errors are
   encountered, it is assumed the file is not really an ASCII file and
   processing for that file is terminated.

   Format:

      % findtab [ file1 [ file2 [ ... ] ] ]

   Input is taken from stdin if no filename is supplied on the command line.

   14 Jan 2000, acr.
   15 Apr 2011: Modified exit() calls to use standard values.
*/

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <errno.h>

#define  MAX_RECORD_LENGTH  15000
#define  MAX_ERRORS         100


main (int argc, char *argv[])

{ int       linecount;    /* line count within current file */
  int       errcount;     /* error count within the current file */
  int       elcount;      /* error count in the current line */
  int       len;          /* string length */
  int       i,j,k;        /* loop/subscript */
  char      line[MAX_RECORD_LENGTH+1]; /* input line buffer */
  int       CC;           /* carriage control flag: 1 = \n, 2 = \r\n */
  FILE     *ifp;          /* Input file pointer */

  /* If no filenames were supplied, insert 'stdin' as the only file: */

  if (argc == 1)
    { argv[1] = (char *)malloc(6);
      strcpy(argv[1], "stdin");
      argc = 2;
    }

  /* Now loop through the input file names: */

  for (i = 1; i < argc; i++)
    { linecount = 0;
      errcount  = 0;

      ifp = fopen(argv[i],"r");
      if (!ifp)
        { fprintf (stderr,"Could not open %s for reading (%s)\n",
                          argv[i],strerror(errno));
          continue;  /* Next file */
        }

      /* OK, now we loop through each line of the file, checking for
         non-printing characters:
      */

      fprintf (stdout,"\n%s --- ",argv[i]);
      while (fgets(line,MAX_RECORD_LENGTH+1,ifp)  &&  errcount <= MAX_ERRORS)
        { len = strlen(line);
          linecount++;

          /* Check for buffer exceeded - which probably means it's not a 
             text file:
          */

          if (len == MAX_RECORD_LENGTH  &&  line[len-1] != '\n')
            { fprintf (stderr,"Maximum record length (%d) exceeded.",
                              MAX_RECORD_LENGTH);
              fprintf (stderr," Text files must have carriage-control.");
              errcount = 999;
              continue;       /* Forces loop exit */
            }

          /* If this is the first line, determine the type of 
             carriage-control:
          */

          if (linecount == 1)
            { CC = (line[len-2] == '\r')? 2 : 1; }

          /* Now we loop through the characters (up to but not including
             the carriage-control characters), looking for non-printing
             bytes:
          */

          elcount = 0;
          for (j=0; j<len-CC; j++)
            { if (!isprint(line[j])) elcount++; }

          /* Now check to see if we found any. If so, report this line: */

          if (elcount > 0)
            { if (errcount == 0)
                { fprintf (stdout, "Non-printing characters found: \n"); }
              errcount += elcount;
              fprintf (stdout,"  Record %d: %d\n", linecount, elcount);
            }

          /* Next line: */

        }

      /* Done with this file.  If there were no errors, report that. Also
         report the presence of \r\n carriage-control:
      */

      fclose(ifp);
      if (errcount == 0)
        { fprintf (stdout,"No non-printing characters found.\n"); }

      if (CC == 2)
        { fprintf (stdout," >>> File has CR/LF carriage-control\n"); }

      /* Next file: */

    }

  /* Done. */

  exit(EXIT_SUCCESS);

}
