/*

    Routine to turn variable-length records into fixed-length records by
    padding with blanks out to a specified length.

    Format:  % fixlen [-c] [-r length] [ input_file [ output_file ] ]

    09 May 1994, acr: re-written to benefit from experience.
    14 Apr 1999, acr: Changed default record length to "79"
    19 Apr 1999, acr: Added "-c" option; changed record length to include all
                      carriage-control characters; change default length to 80
    03 May 2007, acr: Added "-n" (no truncate) option
    07 May 2007, acr: Filled logical hole resulting in first input line 
                      being dropped.
    15 Apr 2011, acr: Modified exit() calls to use standard values.
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>

#define  MAXLENGTH  10000       /* Maximum input record length */
#define  DEFLENGTH     80       /* Default record length       */

main(argc,argv)
  int    argc;
  char  *argv[];

{ int      length;           /* record length */
  FILE    *ifp, *ofp;        /* file pointers */

  char     line[MAXLENGTH];  /* input line buffer */
  char    *rem;              /* string pointer for conversion checking */
  int      i,j,k;            /* loop/subscript */
  int      linelen;          /* input line length */
  int      outlen;           /* output line length */
  int      count;            /* record count */
  int      CR;               /* TRUE if user wants carriage-return characters
                                added; FALSE otherwise */
  int      TRUNC;            /* TRUE if long lines should be truncated */


  length = DEFLENGTH-1;      /* Record length excluding the delimiter \n */
  count  = 0;
  CR     = 0;                /* Default is not to add CR */
  TRUNC  = 1;                /* Default is to truncate long lines */
  ifp    = NULL;
  ofp    = NULL;


  /* Gather up the input arguments and open the files: */

  if (argc > 6)
    { fprintf (stderr,"Usage: fixlen [-c] [-n] [-r length] [ input [ output ] ]\n");
      exit(EXIT_SUCCESS);
    }

  i = 1;
  while (i < argc)
    { if (strcmp(argv[i],"-c") == 0)       /* Add CR option */
          CR = 1;

      else if (strcmp(argv[i],"-n") == 0)  /* No truncate option */
          TRUNC = 0;

      else if (strcmp(argv[i],"-r") == 0)  /* Record length specified */
        { i++;
          length = strtol(argv[i],&rem,10) - 1;
          if (strcmp(rem,"")  ||  length < 1  ||  length > MAXLENGTH)
            { fprintf (stderr,"fixlen: Invalid record length (%s).\n",argv[i]);
              exit(EXIT_FAILURE);
            }
        }

      else if (ifp == NULL)                /* Input file name */
        { if ((ifp = fopen(argv[i],"r")) == NULL)
            { fprintf (stderr,"fixlen: Could not open %s for reading (%s)\n",
                              argv[i],strerror(errno));
              exit(EXIT_FAILURE);
            }
        }

      else                                 /* Output file name */
        { if ((ofp = fopen(argv[i],"w")) == NULL)
            { fprintf (stderr,"fixlen: Could not open %s for writing (%s)\n",
                               argv[i],strerror(errno));
              exit(EXIT_FAILURE);
            }
        }

      i++;     /* Next argument */
    }

  /* Use standard I/O if none was specified: */

  if (ifp == NULL) 
      ifp = stdin;
  if (ofp == NULL) 
      ofp = stdout;

  /* If we're outputting CRLF lines, we'll need an extra space reserved for 
     the CR, so we drop the input length by one:
  */

  if (CR == 1) length--;


/*---------------------------------------------------------------------------*/

  /* Loop through the file records, truncating those which are too long, 
     blank-padding those that are too short.  The record length includes 
     the final carriage control character(s).
  */

  while (fgets(line,MAXLENGTH,ifp) !=NULL) 
    { linelen = strlen(line);
      count++;

      /* We check the first line for carriage control.  If the file currently
         uses CRLF, then we will use CRLF on output regardless of the value 
         of the CR flag:
      */

      if (count == 1  &&  CR == 0)
        { if (line[linelen-2] == '\r') 
            { CR = 1;
              length--;
            }
        }

      /* Check for '\n' character.  If it's not there the record length was
         exceeded:
      */ 

      if (line[linelen-1] != '\n')
        { fprintf (stderr,"fixlen: Maximum line length (%d) exceeded at ",
                           MAXLENGTH);
          fprintf (stderr," record %d.\n",count);
          exit(EXIT_FAILURE);
        }

      /* Erase the current carriage-control by overwriting it with '\0' 
         characters:
      */

      line[--linelen] = '\0';
      if (line[linelen-1] == '\r') line[--linelen] = '\0';
 
      /* Initialize output length to input length (to accommodate
         non-truncation of long lines when the "-n" option is set):
      */

      outlen = length;

      /* Check for short lines: */

      if (linelen < length)
        { for (i=linelen; i < length; ++i) line[i] = ' '; 
          outlen = length;
        }

      /* Check for long lines: */

      else if (linelen > length)
        { fprintf (stderr,"fixlen: output length (%d) exceeded at record %d",
                           length+1+CR, count);

          if (TRUNC)
            { fprintf (stderr,"; truncated.\n");
              outlen = length;
            }

          else
            { fprintf (stderr,".\n");
              outlen = linelen;
            }
        }

      if (CR)
        { line[outlen]   = '\r';
          line[outlen+1] = '\n';
          line[outlen+2] = '\0';
        }
      else
        { line[outlen]   = '\n';
          line[outlen+1] = '\0';
        }

      fputs(line,ofp);
    }

/*---------------------------------------------------------------------------*/

  /* Close files and leave: */

  if (ifp != stdin)  fclose(ifp);
  if (ofp != stdout) fclose(ofp);
  exit(EXIT_SUCCESS);
}
