/*
    inscol:  Routine to insert a column of one or more, possibly non-blank,
             bytes into an ASCII file.  The file is assumed to have
             fixed-length records.

    Format

       inscol <input_file> <byte> [ <string> ]

    where "byte" is the byte count after which the string is to be inserted
    and "string" is the string to be added.  If "string" is omitted a single
    blank character is added.  Strings with special characters must be enclosed
    in quotes.

    Columns will be inserted AFTER the byte number listed in the 
    command line.  Blank columns at the end of record will always be
    inserted BEFORE the linefeed record delimiter.

    23 Mar 1994, acr
    15 Apr 2011, acr: Modified exit() calls to use standard values.

*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#define MAX_RECORD_LENGTH 10000

main (argc,argv)
  int   argc;
  char *argv[];

{ int     i,j,k;                       /* loop/subscript */
  char    inptline[MAX_RECORD_LENGTH]; /* input record */
  char    outline[MAX_RECORD_LENGTH];  /* output record */
  int     column;                      /* column to be inserted */
  char    instring[MAX_RECORD_LENGTH]; /* string to be inserted */
  int     instrlength;                 /* length of input string, sans \0 */
  int     rcount;                      /* record count */
  int     imaxlength;                  /* maximum input record length */
  int     iminlength;                  /* minimum input record length */
  int     omaxlength;                  /* maximum output record length */
  int     ominlength;                  /* minimum output record length */
  int     length;                      /* column list subscript */
  int     c;                           /* column list subscript */
  FILE   *fp;                          /* input file pointer */
  char   *string;                      /* string pointer for length checking */
  int     numc;                        /* number of columnsi n input list */


  /* First, make sure there are at least two arguments, then open
     the input file:
  */

  if (argc < 2  ||  argc > 4)
    { fprintf (stderr,"Usage:  inscol <input file> <column> [<string>]\n");
      exit(EXIT_SUCCESS);
    }

  if ((fp = fopen(argv[1],"r"))==NULL)
    { fprintf (stderr,"inscol: Unable to open %s for reading.\n",argv[1]);
      exit(EXIT_FAILURE);
    }


  /* Get the target column number: */

  if ((column=strtol(argv[2], NULL, 10)) < 0)
    { fprintf (stderr,"inscol: Invalid column number (%s).\n",argv[2]);
      exit(EXIT_FAILURE);
    }


  /* Save input string, if any: */

  if (argc == 4)
    { strcpy(instring,argv[3]);
      instrlength = strlen(instring);
    }
  else
    { instring[0] = ' ';
      instring[1] = '\0';
      instrlength = 1;
    }

/*========================================================================*/

  /* Loop through all records in the file: */

  imaxlength = 0;
  iminlength = MAX_RECORD_LENGTH;
  omaxlength = 0;
  ominlength = MAX_RECORD_LENGTH;
  rcount     = 0;

  while ((string=fgets(inptline,MAX_RECORD_LENGTH,fp)) != NULL)
    { ++rcount;
      length = strlen(string);
      if (length > MAX_RECORD_LENGTH)
         { fprintf (stderr,
                    "inscol: Record %d length (%d) exceeds maximum (%d)\n",
                    rcount, strlen(string), MAX_RECORD_LENGTH);
           exit(EXIT_FAILURE);
         }

      imaxlength = (length > imaxlength) ? length : imaxlength;
      iminlength = (length < iminlength) ? length : iminlength;

      /* Remove the newline (\n) from the end of the input string: */

      --length;
      inptline[length] = '\0';


      /* Check that combined length does not overflow: */

      if ((length + instrlength) > MAX_RECORD_LENGTH)
        { fprintf (stderr,
                  "inscol: Maximum record length (%d) exceeded in record %d.\n",
                  MAX_RECORD_LENGTH, rcount);
          exit(EXIT_FAILURE);
        }


      /* Check that length doesn't underflow.  If so, pad the input string
         with blanks out to the given column number:
      */

      if (length < column)
        { for (i=length; i<column; ++i) inptline[i] = ' ';
          length = column;
        }


      /* Transfer first part of string, insert new string, then transfer
         the last part:
      */

      for (i=0; i<column;      ++i) outline[i]             = inptline[i];
      for (i=0; i<instrlength; ++i) outline[column+i]      = instring[i];
      for (i=column; i<length; ++i) outline[instrlength+i] = inptline[i];
      outline[length+instrlength] = '\0';
      

      /* Write the result to the standard output: */

      length = strlen(outline) + 1;  /* "+1" for the '\n' to be added */
      omaxlength = (length > omaxlength) ? length : omaxlength;
      ominlength = (length < ominlength) ? length : ominlength;
      puts(outline);

    }


  /* Done.  Close file and print results: */

  fclose(fp);
  fprintf (stderr, "%d records processed\n",rcount);
  fprintf (stderr, "  Maximum input length: %5d\n",imaxlength);
  fprintf (stderr, "  Minimum input length: %5d\n",iminlength);
  fprintf (stderr, " Maximum output length: %5d\n",omaxlength);
  fprintf (stderr, " Minimum output length: %5d\n",ominlength);

  exit(EXIT_SUCCESS);
}
