/*
   joinfile:  This routine joins corresponding records from two files into a
              single record in the new output file.  This is most successful
              with fixed-length files of equal numbers of records.  This will
              also work with the results of splitfile.
   Format:

          % joinfile <input 1> <input 2> <output>

   If the files do not have the same number of records, a warning message is
   displayed and the remaining records are appended to the output file.  The
   linefeed is removed from the end of the records in the first input file.

   Modification history:

    19 Nov 1993, acr: creation
    16 Jul 1998, acr: Removed line length restriction
    15 Apr 2011, acr: Modified exit() calls to use standard values.

*/   

#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>


main (argc,argv)
  int   argc;
  char *argv[];
{ 
  FILE *ifd1, *ifd2, *ofd; /* file pointers */
  int   irec1,irec2,orec;  /* record counts */
  int   c1,c2;             /* input characters */

  /* read the arguments: */

  if (argc != 4)
    { printf 
         ("Usage: joinfile <input 1> <input 2> <outfile>\n");
      exit(EXIT_SUCCESS);
    }

  /* Open files: */

  if ((ifd1 = fopen(argv[1],"r")) == NULL)
    { printf("joinfile: Unable to open %s for reading (%s).\n",
              argv[1],strerror(errno));
      exit(EXIT_FAILURE);
    }

  if ((ifd2 = fopen(argv[2],"r")) == NULL)
    { printf("joinfile: Unable to open %s for reading (%s).\n",
              argv[2],strerror(errno));
      exit(EXIT_FAILURE);
    }

  if ((ofd = fopen(argv[3],"w")) == NULL)
    { printf("joinfile: Unable to open %s for writing (%s).\n",
              argv[3],strerror(errno));
      exit(3);
    }


  /* Loop through the input records, joining until one file runs out of 
     lines:
  */

  irec1 = irec2 = orec = 0;

  c1 = fgetc(ifd1);
  c2 = fgetc(ifd2);
  while (c1 != EOF  &&  c2 != EOF)
    { orec++;

      /* Copy characters until the end of record: */

      irec1++;
      while (c1 != '\n'  &&  c1 != EOF)
        { fputc(c1,ofd);
          c1 = fgetc(ifd1);
        }

      /* Read past end of record: */

      if (c1 != EOF)
        { c1 = fgetc(ifd1); }    

      /* Now do the second half of the record: */

      irec2++;
      while (c2 != '\n'  &&  c2 != EOF)
        { fputc(c2,ofd);
          c2 = fgetc(ifd2);
        }

      /* Write the end of the output record and read past it in the input: */

      fputc('\n',ofd);
      if (c2 != EOF)
        { c2 = fgetc(ifd2); }
    }

  /* Now, if we're at the end of file for both files, we're done.  Otherwise,
     we must finish up the remaining file:
  */

  while (c1 != EOF)
    { fputc(c1,ofd);
      c1 = fgetc(ifd1);
      if (c1 == '\n')
        { irec1++; }
    }

  while (c2 != EOF)
    { fputc(c2,ofd);
      c2 = fgetc(ifd2);
      if (c2 == '\n')
        { irec2++; }
    }

  /* Close files: */

  fclose(ofd);
  fclose(ifd1);
  fclose(ifd2);

  exit(EXIT_SUCCESS);
}
