#!/usr/bin/perl

# lblcol
#
#  This routine adds column numbering to the top of an ASCII input file, based
#  on the record length of the first record.  The numbering may be repeated at
#  regular intervals throughout the file, if indicted by the user.
#
#  Format:
#
#    % lblcol [-0] [-r length] [-n <record_count>]   \
#              [ input_file [ output_file ] ]
#
#      Where   -0 [or -O]  indicate zero-based numbers (rather than the PDS
#                          default 1-based)
#              -r length   forces all banner to be "length" bytes
#              -n count    prints a banner every "count" records
#
#  08 Jan 1998, acr.
#  07 Oct 1998, acr: Changed 'require' to use default system libraries
#  19 Jun 2001, acr: Added "-0" and "-r" options. Note that the forced
#                    banner width is accmoplished by resetting local
#                    width variables rather than setting up branching
#                    calls based on the "-r" option value at higher
#                    levels.
#  26 Sep 2006, acr: Modified to use Getopt::Std
#
#==============================================================================

use Getopt::Std;

getopts('n:r:0O');

# NOTE: The "-r" option will be checked as the value of the global variable
# $opt_r. To simplify things for the user, we'll allow the "-0" option to be
# either zero or capital "O", and set an offset base accordingly:

$base = ($opt_0 ||  $opt_O)? 0 : 1;
if ($opt_r  &&  $opt_r < 1) 
  { die "Usage: lblcol [-0] [-r size] [-n count] {input [output]]\n"; }

# If there are filenames on the command line, open them.  Otherwise use 
# standard I/O:

$output_file = (@ARGV > 1)? $ARGV[1] : "-";
$input_file  = (@ARGV > 0)? $ARGV[0] : "-";

open (IN, $input_file)  || die "Unable to open $input_file for input.";
open (OUT,">$output_file") || die "Unable to open $output_file for output.";

# Read the first line from the input file and determine file width:

$line=<IN>;

# Print the first column number banner:

&col_banner(length($line),$base);    # Width includes the newline character
printf OUT $line;

# Now loop through the rest of the file, printing the column banner as
# indicated:

if ($opt_n)
  { $linecount = 1;
    while ($line=<IN>)
      { $linecount++;
        if (($linecount % $opt_n) == 0)
          { &col_banner(length($line),$base); }
        printf OUT $line;
      }
  }

else
  { while ($line=<IN>)
      { printf OUT $line; }
  }

# Close files and we're done:

close(IN);
close(OUT);
   
#_____________________________________________________________________________
#
sub col_banner

{ local ($width,$base) = @_;  # Input width and base (zero or one)
  local ($lines);             # Number of lines in the banner
  local ($level);

  # Determine the number of digits in the width, so that we know how many
  # lines will be in the banner. If we're forcing a banner width, we use
  # the value passed in:

  $lines = ($opt_r)? length($opt_r) : length($width);

  for ($level=$lines; $level>0; $level--)
    { &banner_line($width,$level-1,$base); }
  return 1;
}

#---------------------------------------------------------------------------

sub banner_line

{ local ($width,$power,$base) = @_;

  local ($p10);          # Power of 10;
  local ($end_loop); 
  local ($i,$j);
  local ($digit);

  # Reset $width of we're forcing the banner size:

  $width = ($opt_r)? $opt_r : $width;

  $p10 = 10**$power;
  $end_loop = int($width/$p10);    # Last cycle might be short

  # First iteration might be one short, depending on whether we're 0-based
  # or 1-based:

  for ($i=$base; $i<$p10; $i++)
    { printf OUT "0"; }

  # Middle interations are complete:

  for ($j=1; $j<$end_loop; $j++)
    { for ($i=0; $i<$p10; $i++)
        { printf OUT $j%10; }
    }

  # Last interation ends at end of record. This will write an extra count 
  # in 0-based mode, but it's simpler than doing the math:

  $digit = $width%$p10;
  for ($i=0; $i<=$digit; $i++)
    { printf OUT $end_loop%10; }

  printf OUT "\n";
  return 1;
}
