/*

   scanlen: (Revised) routine to determine the number of lines and the
            max/min byte counts for an ASCII file, without a pre-set
            record length limit.  It also checks for the type of carriage-
            control (CR, LF, or CR/LF), and is not confused by hex nulls
            ('\0') lurking in the file.

   Format:  % scanlen [-r length] [-t] [input_file]

   14 Jan 1997, acr: written to replace existing routine.
   23 Feb 2008, acr: Added -t option to accommodate other routines
   15 Apr 2011, acr: Modified exit() calls to use standard values.

*/

#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>
#include <limits.h>


main(int argc, char *argv[])

{ FILE     *ifp, *ofp;    /* file pointers */
  char      c;            /* input buffer */
  long int  maxlength;    /* input maximum record length */
  long int  maxcount;     /* records longer than maxlength */
  long int  length;       /* length of current input line */
  int       i,j,k;        /* loop/subscript */
  long int  max,min;      /* record length extrema */
  char     *rem;          /* conversion pointer */
  int       ccwarn;       /* carriage control warning flag */
  int       nullwarn;     /* hex null warning flag */
  int       reccount;     /* record count */
  int       showld;       /* TRUE is line delimiters are to be displayed */
  long int  nullcount;    /* number of nulls found */

  ifp       = stdin;
  ofp       = stdout;
  max       = 0;
  min       = LONG_MAX;
  ccwarn    = 0;
  nullwarn  = 0;
  showld    = 0;
  maxlength = 0;
  reccount  = 0;
  maxcount  = 0;
  nullcount = 0;

  
  /* Collect the input arguments and open file as needed: */

  if (argc > 6)
    { fprintf (stderr, "Usage: scanlen [-r length] [input_file]\n");
      exit(EXIT_SUCCESS);
    }

  i = 1;

  while (i < argc)
    { if (strcmp(argv[i],"-r") == 0)
        { i++;
          maxlength = strtol(argv[i],&rem,10);
          if (strcmp(rem,"")  ||  maxlength < 1)
            { fprintf (stderr, "scanlen: Invalid record length, %s\n",argv[i]);
              exit(EXIT_FAILURE);
            }
          i++;
        }
      else if (strcmp(argv[i],"-t") == 0)
        { showld = 1;
          i++;
        }
      else if (ifp == stdin)
        { if ((ifp=fopen(argv[i],"r")) == NULL)
            { fprintf (stderr, "scanlen: Unable to open %s for reading (%s).\n",
                               argv[i], strerror(errno));
              exit(EXIT_FAILURE);
            }
          i++;
        }
      else
        { if ((ofp=fopen(argv[i],"w")) == NULL)
            { fprintf (stderr, "scanlen: Unable to open %s for writing (%s).\n",
                               argv[i], strerror(errno));
              exit(EXIT_FAILURE);
            }
          i++;
        }
    }

/*---------------------------------------------------------------------------*/

  /* Loop through the input file records, one character at a time: */

  length   = 0;

  while ((c=getc(ifp)) != EOF)
    { length++;

      if (c == '\n')
        { max = (length > max)? length : max;
          min = (length < min)? length : min;
          reccount++;
          if (maxlength > 0  &&  length > maxlength)
            { maxcount++;
              fprintf (ofp, "scanlen: Maximum allowed record length ");
              fprintf (ofp, "exceeded at line %d.\n", reccount);
            }

          /* Display the line delimiter if requested and this is the 
             first line:
          */
          if (showld > 0)
            { fprintf (ofp, "scanlen: Line delimiter is LF.\n");
              showld = 0;
            }

          length = 0;
        }

      else if (c == '\r')
        { c = getc(ifp);

          if (c == '\n')
            { length++;
              if (showld == 1)
                { fprintf (ofp,"scanlen: Line delimiter is CR/LF.\n");
                  showld = 0;
                  ccwarn = 1;
                }
              else if (ccwarn == 0)
                { fprintf (ofp,"scanlen: WARNING - file contains CR/LF ");
                  fprintf (ofp,"record delimiters.\n");
                  ccwarn = 1;
                }
            }
          else
            { ungetc(c,ifp);   /* Put the character back to preserve the loop */

              if (showld == 1)
                { fprintf (ofp,"scanlen: Line delimiter is CR.\n");
                  showld = 0;
                  ccwarn = 1;
                }

              else if (ccwarn == 0)
                { fprintf (ofp,"scanlen: WARNING - file contains CR ");
                  fprintf (ofp,"delimiters.\n");
                  ccwarn = 1;
                }
            }
          max = (length > max)? length : max;
          min = (length < min)? length : min;
          reccount++;
          if (maxlength > 0  &&  length > maxlength)
            { maxcount++;
              fprintf (ofp, "scanlen: Maximum record length exceeded ");
              fprintf (ofp, "at line %d.\n", reccount);
            }
          length = 0;
        }

      else if (c=='\0')
        { nullcount++;
          if (nullwarn == 0)
            { fprintf (ofp, "scanlen: WARNING - this file contains ");
              fprintf (ofp, "null characters.\n");
            }
          nullwarn = 1;
        }
    }

  /* Close files and print results: */

  if (ifp != stdin) fclose(ifp);

  fprintf (ofp, "scanlen: %d records checked, maximum length = %d, ",
                   reccount, max);
  fprintf (ofp, "minimum length = %d.\n", min);
  if (maxlength) fprintf (ofp, "         %d records exceeded %d bytes.\n",
                                  maxcount, maxlength);
  if (nullcount) fprintf (ofp, "         %d null characters were found.\n",
                                  nullcount);
  exit(EXIT_SUCCESS);
}
