/*
   splitfile:  This routine splits a single file into two by dividing individual
               records.  The first part of the record goes to the first output
               file, the remainder of the record into the second output file.

   Format:

          % splitfile <input_file> <n_bytes> <output 1> <output 2>

   The first n_bytes from each record are written into the first file, any
   remaining bytes are written into the second.

   The input file need not be of fixed length, but for predictable results it
   should be.

   Modification history:

    18 Nov 1993, acr: creation
    16 Jul 1998, acr: record length restriction removed
    15 Apr 2011, acr: Modified exit() calls to use standard values.

*/   

#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>


main (argc,argv)
  int   argc;
  char *argv[];
{ 
  int   n_bytes;           /* number of bytes in first half of record */
  int   ccount;            /* character count in line */
  int   c;                 /* input character */
  FILE *ifd, *ofd1, *ofd2; /* file pointers */
  int   i,j,k;             /* loop/subscript variables */
  int   length;            /* length of input line */
  char *sp;                /* string pointer used for error checking */
  int   rec;               /* record count */

  /* read the arguments: */

  if (argc != 5)
    { printf 
         ("Usage: splitfile <input file> <n_bytes> <outfile 1> <outfile2>\n");
      exit(EXIT_SUCCESS);
    }

  /* Convert n_bytes to integer: */

  n_bytes = atoi(argv[2]);
  if (errno == ERANGE  ||  n_bytes < 1)
    { printf ("splitfile: Invalid number of bytes (%s)\n", argv[2]);
      exit(EXIT_FAILURE);
    }

  /* Open files: */

  if ((ifd = fopen(argv[1],"r")) == NULL)
    { printf("splitfile: Unable to open %s for reading (%s).\n",
              argv[1],strerror(errno));
      exit(EXIT_FAILURE);
    }

  if ((ofd1 = fopen(argv[3],"w")) == NULL)
    { printf("splitfile: Unable to open %s for writing (%s).\n",
              argv[3],strerror(errno));
      exit(EXIT_FAILURE);
    }

  if ((ofd2 = fopen(argv[4],"w")) == NULL)
    { printf("splitfile: Unable to open %s for writing (%s).\n",
              argv[4],strerror(errno));
      exit(EXIT_FAILURE);
    }


  /* Loop through input records, splitting and writing to the respective
     files:
  */

  rec = 0;
  c = fgetc(ifd);
  while (c != EOF)
    { ++rec;

      /* First half of the line: */

      ccount=0;
      while (ccount < n_bytes  &&  c != '\n'  &&  c != EOF)
        { ccount++;
          fputc(c,ofd1);
          c = fgetc(ifd);
        }
      fputc('\n',ofd1);

      /* End of first half.  If we're at the end of line, we write a null
         second half.  Otherwise, we continue copying characters until we 
         reach either end of line or end of file:
      */

      if (c == '\n')
        { fputc('\n',ofd2); }
      else if (c != EOF)
        { while (c != '\n'  &&  c != EOF)
            { fputc(c,ofd2);
              c = fgetc(ifd);
            }
          fputc('\n',ofd2);   
        }

       /* If we're only at the end of record, read past it and continue: */

       if (c != EOF)
         { c = fgetc(ifd); }
    }

  /* Close files: */

  fclose(ifd);
  fclose(ofd1);
  fclose(ofd2);

  exit(EXIT_SUCCESS);
}
